/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>

#include <qbitmap.h> 
#include <qevent.h>
#include <qimage.h>
#include <qpainter.h> 
#include <qstring.h>
#include <qwidget.h>

#include <kapp.h>
#include <kiconloader.h>

#include "dragsource.h"


CUrlLabel::CUrlLabel( QWidget *parent, const char *name )
  : QLabel( "UrlLabel", parent, name )
{
  mValid = true;
  mDragPending = false;
}


CUrlLabel::~CUrlLabel( void )
{
}

void CUrlLabel::mousePressEvent( QMouseEvent *e )
{
  if( mValid == false || e->button() != LeftButton || mUrl.isEmpty() == true )
  {
    return;
  }
 
  mDragOrigin = e->pos();
  mDragPending = true;
}


void CUrlLabel::mouseMoveEvent( QMouseEvent *e )
{
  if( mDragPending == true )
  {
    if( abs(e->x() - mDragOrigin.x()) + abs(e->y() - mDragOrigin.y()) > 5 )
    {
      mDragPending = false;

      QStrList list;
      list.append( mUrl.data() );

      //
      // Make drag object, assign pixmap and grab keyboard. The grabbing
      // will allow ESC to abort the drag
      //
      QUrlDrag *urlDrag = new  QUrlDrag( list, this );
      prepPixmap( *urlDrag );
      grabKeyboard();
      urlDrag->drag();
      releaseKeyboard();
    }
  }
}

void CUrlLabel::mouseReleaseEvent( QMouseEvent * )
{
  mDragPending = false;
}



void CUrlLabel::setUrl( const QString &url )
{
  mUrl = url;
}

void CUrlLabel::setDragMask( const QPixmap pix )
{
  mDragMask = pix;
}


void CUrlLabel::prepPixmap( QUrlDrag &urlDrag )
{
  if( pixmap() == 0 )
  {
    return;
  }
  
  QString text;
  int index = mUrl.findRev( '/', mUrl.length() );
  if( index < 0 )
  {
    text = mUrl;
  }
  else
  {
    text = mUrl.right( mUrl.length() - index - 1 );
  }

  int sep = 2;
  QRect rect = fontMetrics().boundingRect ( text.data() );
  int w = pixmap()->width() + rect.width() + sep + 1;

  QBitmap mask( w, pixmap()->height(), TRUE );
  QPixmap dragPixmap( w, pixmap()->height() );
  dragPixmap.fill( black );
 
  QPainter p;
  p.begin( &mask );
  p.setPen( white );
  p.drawPixmap( 0, 0, mDragMask );
  p.drawText( pixmap()->width()+sep, 0, w-pixmap()->width()+sep, 
	      pixmap()->height(), AlignVCenter|AlignLeft, text.data() );
  p.end();
  
  p.begin( &dragPixmap );
  p.drawPixmap( 0, 0, *pixmap() );
  p.end();
  
  dragPixmap.setMask( mask );

  QPoint hotspot( pixmap()->width(),pixmap()->height()/2 );
  urlDrag.setPixmap( dragPixmap, hotspot ); 
}



CDropSite::CDropSite( QWidget * parent ) 
  : QObject( parent ), QDropSite( parent )
{
}

void CDropSite::dragMoveEvent( QDragMoveEvent *e )
{
  emit dragMove(e);
}

void CDropSite::dragEnterEvent( QDragEnterEvent *e )
{
  emit dragEnter(e);
}

void CDropSite::dragLeaveEvent( QDragLeaveEvent *e )
{
  emit dragLeave(e);
}

void CDropSite::dropEvent( QDropEvent *e )
{
  emit dropAction(e);
}

