/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _EXPORT_DIALOG_H_
#define _EXPORT_DIALOG_H_

#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h> 
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qwidgetstack.h>

#include <ksimpleconfig.h>

#include "hexbuffer.h"
#include "pagedialog.h"

class CExportDialog : public CPageDialog
{
  Q_OBJECT

  public:
    enum EPage
    {
      page_destination = 0,
      page_option,
      page_max
    };

    enum OptionPage
    {
      option_text = 0,
      option_html
    };

    CExportDialog( const QString &title, QWidget *parent = 0, char *name = 0,
		   bool modal = false );
    ~CExportDialog( void );
    void writeConfiguration( void );

  signals:
    void exportText( const SExportText &e );
    void exportHtml( const SExportHtml &e );

  protected slots:
    virtual void slotHelp( void );
    virtual void slotOk( void );
 
  private:
    struct SDestinationWidgets
    {
      QComboBox    *formatCombo;
      QLabel       *fileExtraLabel;
      QLineEdit    *fileInput;
      QLabel       *fromLabel;
      QLabel       *toLabel;
      QButtonGroup *rangeBox;
      QRadioButton *radio1;
      QRadioButton *radio2;
      QRadioButton *radio3;
      QLineEdit    *fromInput;
      QLineEdit    *toInput;
    };

    struct SOptionWidgets 
    {
      QSpinBox  *lineSpin;
      QLineEdit *prefixInput;
      QComboBox *topCombo;
      QComboBox *bottomCombo;
      QCheckBox *symlinkCheck;
      QCheckBox *navigatorCheck;
      QCheckBox *bwCheck;
    };

  private slots:
    void rangeChanged( int id );
    void formatChanged( int index );
    void browserClicked( void );

  private:
    void setupDestinationPage( void );
    void setupOptionPage( void );
    void makeTextOption( void );
    void makeHtmlOption( void );
    void readConfiguration( void );
    bool collectRange( uint &mode, uint &start, uint &stop );
    bool verifyPackage( const QString &path );

  private:
    QFrame *mFrame[ page_max ];
    QWidgetStack *mOptionStack;
    SDestinationWidgets  mDestination;
    SOptionWidgets mOption;
    KSimpleConfig *mConfig;

    QString mWorkDir;
};





#endif
