/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


//
// getopt.h does not exist on all platforms so be careful to
// use it.
//
//#define USE_GETOPT

#include "toplevel.h"

#ifdef USE_GETOPT
#include <getopt.h>
#endif
#include <iostream.h>
#include <unistd.h>


static uint parseDecimalOrHexadecimal( char *buf );



int main( int argc, char **argv )
{
  KApplication *app = new KApplication( argc, argv, "khexedit" );
  if( app == 0 )
  {
    cerr << "Unable to start - Memory exhausted" << endl;
    return( 1 );
  }

  if( app->isRestored() != 0 )
  {
    RESTORE( KHexEdit );
  }
  else
  {
    KHexEdit *hexEdit = new KHexEdit;
    if( hexEdit == 0 || hexEdit->isValid() == false )
    {
      cerr << "Unable to start - Memory exhausted" << endl;
      return( 1 );
    }

    hexEdit->show();

    #ifndef USE_GETOPT
    if( argc > 1 )
    {
      for( int i=1; i < argc; i++ )
      {
	if( strncmp( argv[i], "-", 1 )==0 || strncmp( argv[i], "--", 2 )==0 )
	{
	  uint k = strncmp( argv[i], "--", 2 ) == 0 ? 2 : 1;
	  if( strcmp( &argv[i][k], "offset" ) == 0 )
	  {
	    if( i+1 < argc )
	    {
	      hexEdit->setStartupOffset( parseDecimalOrHexadecimal(argv[i+1]));
	      i+=1;
	    }
	  }
	}
	else
	{
	  hexEdit->addStartupFile( QString(argv[i]) );
	}
      }
    }
    #endif

    #ifdef USE_GETOPT
    if( argc > 1 )
    {
      static struct option options[] =
      {
	{"offset", 1, 0, 1},
	{0, 0, 0, 0}
      };

      while( 1 )
      {
	int index;
	int optVal = getopt_long_only( argc, argv, "", options, &index );
	if( optVal == -1 ) 
	{ 
	  break; 
	}
	else if( optVal == 1 )
	{
	  hexEdit->setStartupOffset( parseDecimalOrHexadecimal(optarg) );
	  break;
	}
      }
   
      while( optind < argc )
      {
	hexEdit->addStartupFile( QString(argv[optind++]) );
      }
    }
    #endif
  }

  int result = app->exec();
  delete app;
  return( result );
}



static uint parseDecimalOrHexadecimal( char *buf )
{
  if( buf == 0 ) { return( 0 ); }
  
  long int value;
  char *end = 0;
	    
  value = strtol( buf, &end, 10 );
  if( *end != 0 )
  {
    value = strtol( buf, &end, 16 );
    if( *end != 0 )
    {
      value = 0;
    }
  }
  if( value < 0 ) { value = 0; }
  return( value );
}

