/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <qcheckbox.h>
#include <qframe.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h> 
#include <qspinbox.h> 

#include <kapp.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "listview.h"
#include "stringdialog.h"


#define FIRST_COLUMN_SIZE 13


CStringDialog::CStringDialog( QWidget *parent, const char *name, bool modal )
  : KDialogBase( Plain, i18n("Extract Strings"), Help|User1|Cancel, User1,
		 parent, name, modal, true, i18n("&Update") ),
    mBusy(false), mDirty(false) 
{
  setHelp( "khexedit/khexedit.html", QString::null );
 
  QString text;
  QVBoxLayout *topLayout = new QVBoxLayout( plainPage(), 0, spacingHint() );
  if( topLayout == 0 ) { return; }

  QGridLayout *gbox = new QGridLayout( 3, 3, spacingHint() );
  if( gbox == 0 ) { return; }
  topLayout->addLayout( gbox );
  gbox->setColStretch( 1, 10 );
 
  mLengthSpin = new QSpinBox( plainPage() );
  if( mLengthSpin == 0 ) { return; }
  mLengthSpin->setFixedHeight( mLengthSpin->sizeHint().height() );
  mLengthSpin->setMinimumWidth( fontMetrics().maxWidth()*17 );
  mLengthSpin->setRange( 1, 1000000 );
  mLengthSpin->setValue( 4 );
  gbox->addMultiCellWidget( mLengthSpin, 0, 0, 1, 2 );

  text = i18n("&Minimum length");
  QLabel *label = new QLabel( mLengthSpin, text, plainPage() );
  label->setFixedHeight( mLengthSpin->sizeHint().height() );
  label->setFixedWidth( label->sizeHint().width() );
  gbox->addWidget( label, 0, 0, AlignRight );

  mFilter = new QLineEdit( plainPage() );
  if( mFilter == 0 ) { return; }
  mFilter->setFixedHeight( mLengthSpin->sizeHint().height() );
  mFilter->setMinimumWidth( fontMetrics().maxWidth()*17 );
  mFilter->setText("*");
  gbox->addWidget( mFilter, 1, 1 );
  
  text = i18n("&Filter");
  label = new QLabel( mFilter, text, plainPage() );
  label->setFixedHeight( mLengthSpin->sizeHint().height() );
  label->setFixedWidth( label->sizeHint().width() );
  gbox->addWidget( label, 1, 0, AlignRight );

  QPushButton *useButton = new QPushButton( plainPage(), "use" );
  if( useButton == 0 ) { return; }
  useButton->setText( i18n("&Use") );
  useButton->setFixedWidth( useButton->sizeHint().width() );
  connect( useButton, SIGNAL(clicked()),this,SLOT(slotUser1()) );
  gbox->addWidget( useButton, 1, 2 );

  text = i18n("&Ignore case");
  mCheckIngoreCase = new QCheckBox( text, plainPage() );
  mCheckIngoreCase->setMinimumSize( mCheckIngoreCase->sizeHint() );
  gbox->addWidget( mCheckIngoreCase, 2, 1, AlignLeft );

  //
  // Using listview as suggested by Dima Rogozin <dima@mercury.co.il>
  //
  mStringList = new CListView( plainPage(), "stringList" );
  mStringList->setFont( KGlobal::fixedFont() ); 
  mStringList->addColumn( i18n("Offset") );
  mStringList->addColumn( i18n("String") );
  mStringList->setAllColumnsShowFocus( true );
  mStringList->setFrameStyle( QFrame::WinPanel + QFrame::Sunken );

  connect( mStringList, SIGNAL(selectionChanged()),
	   SLOT(selectionChanged()));
  topLayout->addWidget( mStringList, 10 );
  
  QHBoxLayout *hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );
  
  text = i18n("Number of strings:");
  label = new QLabel( text, plainPage() );
  label->setFixedHeight( label->sizeHint().height() );
  label->setFixedWidth( label->sizeHint().width() );
  hbox->addWidget( label, AlignLeft );
  
  mListSizeLabel = new QLabel( QString(), plainPage() );
  mListSizeLabel->setFixedWidth( fontMetrics().width("8")*10 );
  hbox->addWidget( mListSizeLabel, AlignLeft|AlignHCenter );

  text = i18n("Displayed:");
  label = new QLabel( text, plainPage() );
  label->setFixedHeight( label->sizeHint().height() );
  label->setFixedWidth( label->sizeHint().width() );
  hbox->addWidget( label, AlignLeft );
  
  mDisplaySizeLabel = new QLabel( QString(""), plainPage() );
  mDisplaySizeLabel->setFixedWidth( fontMetrics().width("8")*10 );
  hbox->addWidget( mDisplaySizeLabel, AlignLeft|AlignHCenter);
  
  hbox->addStretch( 10 );

  mDirtyLabel = new QLabel( plainPage() );
  mDirtyLabel->setFixedHeight( fontMetrics().height() );
  topLayout->addWidget( mDirtyLabel );

  //
  // The initial width of the columns are set in updateListInfo()
  //
  updateListInfo();
  mStringList->setVisibleItem( 15 );
  startTimer(100);
}

	
CStringDialog::~CStringDialog( void )
{
}


void CStringDialog::slotUser1( void ) // Start
{
  if( mBusy == true )
  {
    return;
  }

  mRegExp.setCaseSensitive( mCheckIngoreCase->isChecked()==true ? false:true );
  mRegExp.setWildcard( true );
  QString str = mFilter->text();
  if( str.isEmpty() == true )
  {
    str = "*";
  }

  mRegExp = str;
  if( mRegExp.isValid() == false )
  {
    QString msg = i18n(""
      "The filter expression you have specified is illegal.\n"
      "You must specify a valid regular expression.\n\n"
      "Proceed with no filter?");
    int reply = KMessageBox::warningYesNo( this, msg, i18n("Extract strings"));
    if( reply != KMessageBox::Yes )
    {
      return;
    }
    mRegExp = "*";
  }


  mStringData.minLength = mLengthSpin->value();

  removeList();

  mBusy = true;
  emit collect();
  mBusy = false;
}


void CStringDialog::startGoto( QListViewItem *item )
{
  const char *str_offset = item->text(0);
  const char *string = item->text(1);

  uint offset,val1,val2;
  sscanf( str_offset, "%X:%X", &val1, &val2 );
  offset = (val1<<16) + val2;
  
  uint size = strlen( string );
  
  emit markText( offset, size, true );
}


void CStringDialog::setDirty( void )
{
  if( mDirty == true )
  {
    return;
  }
  
  mDirtyLabel->setText(
    i18n("Warning: Document has been modified since last update"));
  mDirty = true;
}


void CStringDialog::setClean( void )
{
  if( mDirty == false )
  {
    return;
  }
  
  mDirtyLabel->setText("");
  mDirty = false;
}


void CStringDialog::selectionChanged()
{
  killTimers();
  startTimer( 200 );
}


void CStringDialog::timerEvent( QTimerEvent * )
{
  killTimers();

  QListViewItem *item = mStringList->currentItem();
  if( item == 0 )
  {
    slotUser1();
    return;
  }
  startGoto( item );
}


void CStringDialog::resizeEvent( QResizeEvent * )
{
  setColumnWidth();
}



int CStringDialog::setList( QList<QString> &list, CProgress &p )
{
  clearList();
  enableList( false );

  for( uint i=0; i < list.count(); i++ )
  {
    const QString *str = list.at(i);
    if( str != 0 )
    {
      appendListItem( *str );
    }

    if( (i+1) % 200 == 0 )
    {
      updateListInfo();

      int errCode = p.step( (float)i/(float)list.count() );
      if( errCode == Err_Stop )
      {
	p.finish();
	enableList( true );
	return( errCode );
      }
    }
  }

  p.finish();
  updateListInfo();
  enableList( true );

  return( Err_Success );
}


void CStringDialog::clearList( void )
{
  if( mStringList->childCount() > 0 )
  {
    mStringList->clear();
    mStringList->update();
  }
  updateListInfo();
  setClean();
}


void CStringDialog::removeList( void )
{
  mStringData.clear();
  clearList();
}


void CStringDialog::enableList( bool state )
{
  mStringList->setEnabled( state );
  if( state == true )
  {
    mStringList->update();
  }
}


void CStringDialog::appendListItem( const QString &str )
{
  if( mRegExp.match( str, 13 ) != -1 )
  {
    new QListViewItem(mStringList,
		      str.left(11), str.mid(13, 13 + str.length()));
  }
}


void CStringDialog::updateListInfo( void )
{
  mListSizeLabel->setText( QString("%1").arg(mStringData.count()) );
  mDisplaySizeLabel->setText( QString("%1").arg(mStringList->childCount()) );
  setColumnWidth();
}



void CStringDialog::setColumnWidth( void )
{
  if( mStringData.count() == 0 )
  {
    int w0 = mStringList->fontMetrics().maxWidth() * FIRST_COLUMN_SIZE;
    int w1 = mStringList->viewport()->width() - w0;
    mStringList->setColumnWidth( 0, w0 );
    mStringList->setColumnWidth( 1, w1 );

    //
    // If the list is emptied while the dialog box is larger than its 
    // original size then it seems we should resize the viewport as well
    // (Qt-1.44). But it doesn't work. The viewport seems unchanged.
    //
    /*
    mStringList->viewport()->resize( 
      mStringList->columnWidth(0)+mStringList->columnWidth(1),
      mStringList->viewport()->height() );     
    mStringList->update();
    */
  }
}


